#!ruby -Ku
# -*- coding: utf-8 -*-
#
#  TOPPERS Configurator by Ruby
#
#  Copyright (C) 2015 by Embedded and Real-Time Systems Laboratory
#              Graduate School of Information Science, Nagoya Univ., JAPAN
#  Copyright (C) 2015 by FUJI SOFT INCORPORATED, JAPAN
#
#  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
#  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
#  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
#  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
#      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
#      スコード中に含まれていること．
#  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
#      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
#      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
#      の無保証規定を掲載すること．
#  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
#      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
#      と．
#    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
#        作権表示，この利用条件および下記の無保証規定を掲載すること．
#    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
#        報告すること．
#  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
#      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
#      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
#      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
#      免責すること．
#
#  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
#  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
#  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
#  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
#  の責任を負わない．
#
#  $Id: SRecord.rb 46 2016-02-09 16:19:46Z ertl-hiro $
#

######################################################################
# Sレコードファイル処理クラス定義
#
#
# @hSrecDataは，先頭番地をキーとし，そこからのデータ（ヘキサダンプ形式）
# を値とブロックが格納されたハッシュである．連続するブロックは，1つのブ
# ロックにまとめて格納するものとする（そうしないと，データの取出しやサー
# チが面倒になる）．
######################################################################
class SRecord
  def initialize(sFile)
    @hSrecData = {}
    File.open(sFile) do |cFile|
      nPrevAddress = 0
      sPrevData = ""
      cFile.each do |sLine|
        # レコードタイプ
        sType = sLine.slice(0, 2)

        # データレコードのみ処理する
        case sType
        when "S1"
          # データ長(アドレス分[2byte]+チェックサム分1byteを減算)
          nLength = sLine.slice(2, 2).hex() - 2 - 1

          # アドレス(4文字=2byte)
          nAddress = sLine.slice(4, 4).hex()

          # データ(この時点では文字列で取っておく)
          sData = sLine.slice(8, nLength * 2)
        when "S2"
          # データ長(アドレス分[3byte]+チェックサム分1byteを減算)
          nLength = sLine.slice(2, 2).hex() - 3 - 1

          # アドレス(6文字=3byte)
          nAddress = sLine.slice(4, 6).hex()

          # データ(この時点では文字列で取っておく)
          sData = sLine.slice(10, nLength * 2)
        when "S3"
          # データ長(アドレス分[4byte]+チェックサム分1byteを減算)
          nLength = sLine.slice(2, 2).hex() - 4 - 1

          # アドレス(8文字=4byte)
          nAddress = sLine.slice(4, 8).hex()

          # データ(この時点では文字列で取っておく)
          sData = sLine.slice(12, nLength * 2)
        else
          nAddress = nil
        end

        if !nAddress.nil?
          # データを格納する
          if nAddress == nPrevAddress + sPrevData.size / 2
            sPrevData += sData
          else
            set_data(nPrevAddress, sPrevData) if sPrevData.size > 0
            nPrevAddress = nAddress
            sPrevData = sData
          end
        end
      end
      set_data(nPrevAddress, sPrevData) if sPrevData.size > 0
    end
  end

  # データ取得
  def get_data(nAddress, nSize)
    nEndAddress = nAddress + nSize
    @hSrecData.each do |nBlock, sBlockData|
      if (nBlock <= nAddress && nEndAddress <= nBlock + sBlockData.size / 2)
        nOffset = (nAddress - nBlock) * 2
        return(sBlockData[nOffset, nSize * 2])
      end
    end
    return(nil)
  end

  # データ書込み
  def set_data(nAddress, sData)
    nEndAddress = nAddress + sData.size / 2
    @hSrecData.each do |nBlock, sBlockData|
      nEndBlock = nBlock + sBlockData.size / 2

      if nEndBlock < nAddress || nEndAddress < nBlock
        # 重なりがない
      elsif nAddress < nBlock
        # 新規データの方が先頭番地が小さい
        # ここでは nEndAddress >= nBlock が成立している
        nOffset = (nEndAddress - nBlock) * 2
        sData += sBlockData[nOffset..-1]
        @hSrecData.delete(nBlock)
      else
        # 登録済みデータの方が先頭番地が小さいか同じ
        nOffset = (nAddress - nBlock) * 2
        nAddress = nBlock
        sBlockData[nOffset, sData.size] = sData
        sData = sBlockData
        @hSrecData.delete(nBlock)
      end
    end
    @hSrecData[nAddress] = sData
  end

  # 値としてのデータ取得
  def get_value(nAddress, nSize, bSigned)
    sTargetData = get_data(nAddress, nSize)
    if sTargetData.nil?
      return(nil)
    else
      if $bLittleEndian == true
        # リトルエンディアンの場合，バイトオーダーを逆にする
        sReverseData = ""
        # 後ろから2文字ずつ抜き出し，並び替える
        while sTargetData.size > 0
          sReverseData += sTargetData[-2, 2]
          sTargetData.slice!(-2, 2)
        end
        nReturnData = sReverseData.hex
      else
        nReturnData = sTargetData.hex
      end

      if bSigned == true && (nReturnData & (1 << (nSize * 8 - 1))) != 0
        nReturnData -= (1 << (nSize * 8))
      end
      return(nReturnData)
    end
  end
end
